/*-------------------------------------------------------------*\
 | CHARVIEW.CPP  Dr. Dobbs Journal Sample Application.         |
 |                                                             |
 | By: Paul Yao                                                |
 |     International Systems Design                            |
 |                                                             |
 | Purpose: Demonstrate Borland C++ 3.1 and ObjectWindows      |
 |          Library (OWL) Application Framework.               |
 |                                                             |
\*-------------------------------------------------------------*/
#define WIN31
#define STRICT

#include <Owl.H>
#include <WindowsX.h>
#include <Stdio.H>
#include <String.H>

#include <filedial.h>
#include <listbox.h>

extern "C" {
#include "Chv.H"
};

PUBLIC BYTE   chCurrent = 'A';
PUBLIC BYTE   chErrBuf[MAX_CHARS];
PUBLIC int    inCurrent  = 0;
PUBLIC LPLIST CharData[MAX_CHARS];

#include "Classes.H"

PTListBox ptList;

/*-------------------------------------------------------------*\
 |                 Main Function:  WinMain.                    |
\*-------------------------------------------------------------*/
int PASCAL
WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
         LPSTR  lpszCmdLine,  int       nCmdShow)
    {
    TCVApplication Viewer ("Viewer", hInstance, hPrevInstance,
                           lpszCmdLine, nCmdShow);

    if (*lpszCmdLine != '\0')
        {
        file_LoadHWXData(lpszCmdLine);
        }

    Viewer.Run();
    return Viewer.Status;
    }

/*-------------------------------------------------------------*\
 |                  Application Class Member.                  |
\*-------------------------------------------------------------*/
TCVApplication::TCVApplication (LPSTR lpszName, HINSTANCE hInstance,
                    HINSTANCE hPrevInstance, LPSTR lpszCmdLine,
                    int nCmdShow)
                :TApplication (lpszName, hInstance, hPrevInstance,
                    lpszCmdLine, nCmdShow)
    {
    /*  Application specific initialization goes here.  */
    }

/*-------------------------------------------------------------*\
 |                  Application Class Member.                  |
\*-------------------------------------------------------------*/
void
TCVApplication::InitMainWindow ()
    {
    MainWindow = new TMainWindow (NULL, "Character Viewer", NULL);
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
TMainWindow::TMainWindow (PTWindowsObject pwParent,
               LPSTR lpszTitle, PTModule pmModule)
           :TWindow (pwParent, lpszTitle, pmModule)
    {
    /*  Window specific initialization goes here.  */
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
TMainWindow::~TMainWindow()
    {
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
LPSTR
TMainWindow::GetClassName ()
    {
    return "CHARVIEW:MAIN";
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void
TMainWindow::GetWindowClass (WNDCLASS& wc)
    {
    TWindow::GetWindowClass (wc);
    wc.hIcon        = LoadIcon (wc.hInstance, "snapshot");
    wc.hCursor      = LoadCursor (wc.hInstance, "hand");
    wc.lpszMenuName = MAKEINTRESOURCE (IDR_MAINMENU);
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void
TMainWindow::CmdFileOpen(TMessage&)
    {
    char        achFile[MAXFILENAME];
    int         iRetVal;
    TFileDialog * ptFileOpen;

    // Set up wild-card & file path information.
    _fstrcpy (achFile, "*.*");

    // Create a dialog object
    ptFileOpen = new TFileDialog(this, SD_FILEOPEN, achFile,
                                 NULL);

    // Display File.Open dialog
    iRetVal = GetApplication()->ExecDialog (ptFileOpen);

    // If user clicked 'Ok', open file. Routine also frees
    // previous file's data structures.
    if (iRetVal == IDOK)
        {
        file_LoadHWXData(achFile);
        PostMessage (hwndList, WM_CHAR, (WPARAM)(BYTE)'A', 0L);
        InvalidateRect (ptLeft->HWindow, NULL, TRUE);
        InvalidateRect (ptRight->HWindow, NULL, TRUE);
        }
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void
TMainWindow::CmdScale50(TMessage&)
    {
    HMENU hmenu;

    hmenu = GetMenu (HWindow);
    CheckMenuItem (hmenu, IDM_SCALE_50, MF_BYCOMMAND|MF_CHECKED);
    CheckMenuItem (hmenu, IDM_SCALE_75, MF_BYCOMMAND|MF_UNCHECKED);
    CheckMenuItem (hmenu, IDM_SCALE_100, MF_BYCOMMAND|MF_UNCHECKED);

    ptLeft-> SetScalingFactor(50);
    ptRight->SetScalingFactor(50);
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void
TMainWindow::CmdScale75(TMessage&)
    {
    HMENU hmenu;

    hmenu = GetMenu (HWindow);
    CheckMenuItem (hmenu, IDM_SCALE_50, MF_BYCOMMAND|MF_UNCHECKED);
    CheckMenuItem (hmenu, IDM_SCALE_75, MF_BYCOMMAND|MF_CHECKED);
    CheckMenuItem (hmenu, IDM_SCALE_100, MF_BYCOMMAND|MF_UNCHECKED);

    ptLeft-> SetScalingFactor(75);
    ptRight->SetScalingFactor(75);
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void
TMainWindow::CmdScale100(TMessage&)
    {
    HMENU hmenu;

    hmenu = GetMenu (HWindow);
    CheckMenuItem (hmenu, IDM_SCALE_50, MF_BYCOMMAND|MF_UNCHECKED);
    CheckMenuItem (hmenu, IDM_SCALE_75, MF_BYCOMMAND|MF_UNCHECKED);
    CheckMenuItem (hmenu, IDM_SCALE_100, MF_BYCOMMAND|MF_CHECKED);

    ptLeft-> SetScalingFactor(100);
    ptRight->SetScalingFactor(100);
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void
TMainWindow::CmdListboxNotify(TMessage& Msg)
    {
    int  iItem;

    if (HIWORD (Msg.LParam) == LBN_SELCHANGE)
        {
        iItem = ListBox_GetCurSel(hwndList);
        chCurrent = (BYTE)iItem + FIRST_CHAR;
        if (chCurrent == 0x01) chCurrent = 'A';
        inCurrent  = 0;
        ptLeft->SetCurrentChar (chCurrent);
        ptRight->SetCurrentChar (chCurrent);
        }
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void TMainWindow::SetupWindow ()
    {
    BYTE       ach[4];
    HDC        hdc;
    int        cxColumn;
    TEXTMETRIC tm;

    TWindow::SetupWindow ();

#if 1  // OWL approach to window creation...

    ptList  = new TListBox (this, IDC_LISTBOX, 0, 0, 0, 0, NULL);
    ptList->Attr.Style = WS_CHILD | WS_BORDER | WS_HSCROLL |
                         WS_VISIBLE | LBS_NOTIFY |
                         LBS_NOINTEGRALHEIGHT | LBS_MULTICOLUMN;
    GetApplication()->MakeWindow (ptList);
    hwndList = ptList->HWindow;

#else  // ...versus native Windows API approach.

    hwndList = CreateWindowEx (WS_EX_NOPARENTNOTIFY,
                               "listbox",
                               NULL,
                               WS_CHILD   | WS_BORDER  |
                               WS_HSCROLL | WS_VISIBLE |
                               LBS_NOINTEGRALHEIGHT    |
                               LBS_NOTIFY  |
                               LBS_MULTICOLUMN,
                               0,
                               0,
                               0,
                               0,
                               HWindow,
                               (HMENU)IDC_LISTBOX,
                               GetApplicationObject()->hInstance,
                               NULL);
#endif

    hdc = GetDC (NULL);
    GetTextMetrics (hdc, &tm);
    ReleaseDC (NULL, hdc);

    cyLBHeight = (GetSystemMetrics (SM_CYBORDER) * 2 ) +
                  GetSystemMetrics (SM_CYHSCROLL) +
                  tm.tmHeight;

    cxColumn = tm.tmAveCharWidth * 2;

    ListBox_SetColumnWidth (hwndList, cxColumn);

    // Add items to listbox.
    for (ach[0] = FIRST_CHAR, ach[1] = '\0';
         ach[0] != 0; ach[0]++)
        ListBox_AddString (hwndList, &ach[0]);

    // Set initial selection in listbox.
    PostMessage (hwndList, WM_CHAR, (WPARAM)(BYTE)'A', 0L);

    // Create left-side (single instance) viewer window.
    ptLeft  = (PTCharViewWindow)GetApplication()->MakeWindow (new
              TCharViewWindow (this, "", NULL, 100,
                               1, 1, 0, 0, cyLBHeight) );
    // Create right-side (multi instance) viewer window.
    ptRight = (PTCharViewWindow)GetApplication()->MakeWindow (new
              TCharViewWindow (this, "", NULL, 100,
                               4, 5, 0, 15, cyLBHeight) );
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void
TMainWindow::WMActivate(TMessage& Msg)
    {
    // If we're being activated, give keyboard to list box.
    if (Msg.WParam)
        {
        SetFocus (hwndList);
        }
    }

/*-------------------------------------------------------------*\
 |                  TMainWindow Class Member.                  |
\*-------------------------------------------------------------*/
void TMainWindow::WMSize (TMessage &Msg)
    {
    int x, y, cx, cy;
    int cxBorder, cyBorder;
    int cxWindow, cyWindow;

    cxWindow = LOWORD(Msg.LParam);
    cyWindow = HIWORD (Msg.LParam);
    cxBorder = GetSystemMetrics (SM_CXBORDER);
    cyBorder = GetSystemMetrics (SM_CYBORDER);

    x  = 0 - cxBorder;
    y  = cyWindow - cyLBHeight + cyBorder;
    cx = cxWindow + (cxBorder * 2);
    cy = cyLBHeight;
    SetWindowPos (hwndList, (HWND)0, x, y, cx, cy, 0);

    x  = 0 - cxBorder;
    y  = 0 - cyBorder;
    cx = MulDiv (cxWindow, 1, 4) + cxBorder;
    cy = cyWindow - cyLBHeight + (cxBorder * 3);
    SetWindowPos (ptLeft->HWindow, (HWND)0, x, y, cx, cy, 0);

    x  = MulDiv (cxWindow, 1, 4) - cxBorder;
    y  = 0 - cyBorder;
    cx = MulDiv (cxWindow, 3, 4) + (cxBorder * 2);
    cy = cyWindow - cyLBHeight + (cxBorder * 3);
    SetWindowPos (ptRight->HWindow, (HWND)0, x, y, cx, cy, 0);
    }
