#ifndef  __GRIDVIEW_H
#define  __GRIDVIEW_H

#include <owl.h>
#include <scrollba.h>
#include "drawdata.h"

_CLASSDEF( TGridWindow )

class  TGridWindow : public TControl
{

    public:
        enum ViewMode{ gridMode, blownUpMode };   // Window has two
        ViewMode vMode;                           // Viewing Modes

        HPEN hSolidPen;
        BOOL fInFocus;

        //
        // Variables used while in gridMode
        //
        WORD wCellWidth, wCellHeight;        // Char Cell dimensions
	    WORD wVisibleRows, wVisibleCols;     // Number of Visibles Cells
	    WORD wNumRows, wNumCols;	         // Total number of Cells
        WORD wLeftCol, wTopRow;              // Leftmost and Topmost
        WORD wBdrWidth,wBdrHeight;           // Grid Border in Window

        WORD wSelectedCell; /* Col | ( Row << 8 ) */

        // cur char, inst, colors
        PTDrawData pDrawData;


    public:
        TGridWindow( PTWindowsObject AParent, int AnId, PTDrawData pd,
                     PTModule AModule = NULL );
        virtual ~TGridWindow();
        virtual void  Paint( HDC PaintDC, PAINTSTRUCT _FAR & PaintInfo );
        virtual void  SetupWindow( void );
        virtual LPSTR GetClassName(){  return "GridWindow";  };
        virtual void  GetWindowClass( WNDCLASS & AWndClass );


        void UpdateView()
        {
            InvalidateRect( HWindow, NULL, TRUE );
        }


        WORD MAKECELL( int col, int row )
        {
            return ((WORD)(((BYTE)(col)) | (((WORD)((BYTE)(row))) << 8)));
        }
        BYTE CELLCOL( WORD cell ){  return LOBYTE(cell);  }
        BYTE CELLROW( WORD cell ){  return HIBYTE(cell);  }
        WORD CellFromPoint( POINT );
	    void GetCellRect( WORD, RECT& );
	    void SelectNewCell( WORD );
        BOOL IsCellVisible( WORD );

        int  GetCurInst( void );

        void SetNumRows( WORD );
        void SetNumCols( WORD );
        void SetVisibleRows( WORD );
        void SetVisibleCols( WORD );

	    WORD CellLeft();
	    WORD CellRight();
	    WORD CellUp();
        WORD CellDown();

        void ScrollRight();
        void ScrollLeft();
        void ScrollUp();
        void ScrollDown();

        void DrawGrid( HDC );
        void UpdateCells( HDC );
	    void FrameCell( HDC, WORD );
	    void EnableHScroll( BOOL );
	    void EnableVScroll( BOOL );
        
        void GridMode( void );
        void BlownUpMode( void );

        void WMLButtonDown( RTMessage ) = [WM_FIRST  + WM_LBUTTONDOWN];
        void WMLButtonDblClk( RTMessage)= [WM_FIRST  + WM_LBUTTONDBLCLK];
        void WMKeyDown( RTMessage )     = [ WM_FIRST + WM_KEYDOWN ];
        void WMChar( RTMessage )        = [ WM_FIRST + WM_CHAR ];

        void WMGetDlgCode( RTMessage )  = [ WM_FIRST + WM_GETDLGCODE ];
        void WMEraseBkgnd( RTMessage )  = [ WM_FIRST + WM_ERASEBKGND ];
        void WMSetFocus( RTMessage )    = [ WM_FIRST + WM_SETFOCUS   ];
};



#endif   // __GRIDVIEW_H
