// hwb.cpp : HandWriting Browser for Windows using 
// Microsoft Foundation Classes (MFC)

#include <afxwin.h> 
#include <afxdlgs.h>

#include "hwb.h"

#define DISPLAY	CDC		// a DISPLAY is a CDC object
#include "hwx.h"		// interface functions from Dr. Dobbs

/////////////////////////////////////////////////////////////////////////////
// CHwbDlg

// Message map defines messages handled by our main window
BEGIN_MESSAGE_MAP(CHwbDlg, CModalDialog)
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	ON_WM_MENUSELECT()
	ON_COMMAND(IDM_OPEN, OnOpen)
	ON_COMMAND(IDM_COPY, OnCopy)
	ON_COMMAND(IDM_PRINT, OnPrint)
	ON_COMMAND(IDM_ABOUT, OnAbout)
	ON_COMMAND(IDC_PREV, OnPrevChar)
	ON_COMMAND(IDC_NEXT, OnNextChar)
	ON_COMMAND(IDM_LINE_COLOR, OnLineColor)
#ifdef _DEBUG	// debug commands
	ON_COMMAND(IDM_DEBUG_SHOW_STROKES, OnShowStrokes)
#endif //_DEBUG
	ON_CBN_SELCHANGE(IDC_INSTANCE_BOX, OnInstanceSelect)
	ON_CBN_SELCHANGE(IDC_ALLCHARS_BOX, OnCharSelect)
END_MESSAGE_MAP()


CHwbDlg::CHwbDlg()
	 : CModalDialog("MAINDIALOG"),
		m_brBack(::GetSysColor(COLOR_WINDOW)),
		m_brGray(RGB(128, 128, 128))
{
	m_bLoaded = FALSE;
	m_colorCur = RGB(0, 0, 0);	// black to start with
	m_nLineWidth = 1;
}

CHwbDlg::~CHwbDlg()
{
	if (m_bLoaded)
		::UnloadHWXData();		// free old data file
}

BOOL CHwbDlg::OnInitDialog()
{
	// initialize bitmap buttons
	for (int i = 0; i < NUM_BITMAPBUTTONS; i++)
		VERIFY(m_buttons[i].AutoLoad(IDC_BITMAPBUTTON_MIN + i, this));

	// wire in the dialog controls
	VERIFY(m_imageBox.SubclassDlgItem(IDC_INSTANCE_BOX, this));
	m_imageBox.m_pParent = this;
	VERIFY(m_charBox.SubclassDlgItem(IDC_ALLCHARS_BOX, this));
	m_charBox.m_pParent = this;
	VERIFY(m_scrollBar.SubclassDlgItem(IDC_SCROLLBAR, this));

	// select a lighter font for the status bar
	LOGFONT logfont;
	GetFont()->GetObject(sizeof(logfont), &logfont);
	logfont.lfWeight = FW_NORMAL;
	VERIFY(m_statusFont.CreateFontIndirect(&logfont));
	GetDlgItem(IDC_STATUS)->SetFont(&m_statusFont);

	OnOpen();		// require an open data file

	if (!m_bLoaded)
		EndDialog(IDCANCEL);		// abort

	return TRUE;
}

void CHwbDlg::OnOpen()
{
	CFileDialog	dlg(TRUE, "dat", NULL,
		OFN_FILEMUSTEXIST | OFN_HIDEREADONLY,
		"Data Files (*.dat)|*.dat|All Files (*.*)|*.*||");
		// constructor for standard file open dialog
	
	if (dlg.DoModal() != IDOK)
		return;					// stay with old data file

	HCURSOR hCurs = ::SetCursor(::LoadCursor(NULL, IDC_WAIT));
								// set the wait cursor

	if (m_bLoaded)
		::UnloadHWXData();		// free old data file

	// open new data file
	::LoadHWXData(dlg.GetPathName());
	m_bLoaded = TRUE;

	// fill listbox with all valid characters using exported Dr. Dobbs APIs
	m_charBox.ResetContent();
	for (char ch = 0; ch <= 126; ch++)		// simple ASCII range
	{
		if (::GetInstanceCount(ch) == 0)
			continue;		// skip characters with no instances
		int nIndex = m_charBox.AddString(MAKEINTRESOURCE(ch));
		if (ch == 'A')
			m_charBox.SetCurSel(nIndex);		// initial selection
	}
	OnCharSelect();		// update instances

	::SetCursor(hCurs); // change cursor back to standard
}

void CHwbDlg::OnCopy()	
{
	// copy current image to clipboard

	if (!OpenClipboard())
		return;
	CMetaFileDC dc;
	if (dc.Create())
	{
		RenderInstance(&dc, m_chCur, m_iInstanceCur, CRect(0,0,0,0), 0, TRUE);
		HGLOBAL hData = ::GlobalAlloc(GPTR, sizeof(METAFILEPICT));
		if (hData != NULL)
		{
			LPMETAFILEPICT lpMFP = (LPMETAFILEPICT)::GlobalLock(hData);
			lpMFP->mm = MM_ANISOTROPIC;
			lpMFP->hMF = dc.Close();
			VERIFY(EmptyClipboard());
			::GlobalUnlock(hData);
			SetClipboardData(CF_METAFILEPICT, hData);
		}
		VERIFY(CloseClipboard());
	}
}

void CHwbDlg::OnPrint()
{
	// print the current image

	CPrintDialog dlg(FALSE, PD_NOPAGENUMS | PD_NOSELECTION);

	if (dlg.DoModal() != IDOK)
		return;

	HCURSOR hCurs = ::SetCursor(::LoadCursor(NULL, IDC_WAIT));
								// set the wait cursor

	CDC dc;
	dc.Attach(dlg.GetPrinterDC());
	CRect rect;
	dc.GetClipBox(rect);

	dc.StartDoc("HandWriting");
	dc.StartPage();
	RenderInstance(&dc, m_chCur, m_iInstanceCur, rect, 0, FALSE);
	dc.EndPage();
	dc.EndDoc();

	::SetCursor(hCurs); // return to standard cursor
}

void CHwbDlg::OnAbout()
{
	CModalDialog dlg("ABOUTDIALOG");
	dlg.DoModal();
}

void CHwbDlg::OnNextChar()
{
	m_charBox.SetCurSel(m_charBox.GetCurSel() + 1);
	OnCharSelect();
}

void CHwbDlg::OnPrevChar()
{
	m_charBox.SetCurSel(m_charBox.GetCurSel() - 1);
	OnCharSelect();
}

void CHwbDlg::OnCharSelect()
{
	ASSERT(m_bLoaded);

	if (m_charBox.GetCurSel() == -1)
		m_charBox.SetCurSel(0);		// select first one
	m_chCur = (char)m_charBox.GetItemData(m_charBox.GetCurSel());
	// update the character indicator and box
	SetDlgItemText(IDC_CHARACTER, CString(m_chCur));

	m_nInstanceCount = ::GetInstanceCount(m_chCur);
	m_scrollBar.SetScrollRange(0, m_nInstanceCount-1);

	// update the list of possible instances
	m_imageBox.ResetContent();
	for (int i = 0; i < m_nInstanceCount; i++)
		m_imageBox.InsertString(i, NULL);		// data not used

	m_imageBox.SetCurSel(m_iInstanceCur = 0);	// start at first one
	OnInstanceSelect();		// initial update
}

void CHwbDlg::OnInstanceSelect()
{
	int iSel = m_imageBox.GetCurSel();
	if (iSel != -1)
	{
		ASSERT(iSel >= 0 && iSel < m_nInstanceCount);
		m_iInstanceCur = iSel;
	}

	// update output
	ASSERT(m_bLoaded);
	ASSERT(m_iInstanceCur >= 0 && m_iInstanceCur < m_nInstanceCount);
	m_scrollBar.SetScrollPos(m_iInstanceCur);

	InvalidateRect(GetOutputRect(), FALSE);	// will redraw at paint time
}

/////////////////////////////////////////////////////////////////////////////
// Encapsulated drawing code

CRect CHwbDlg::GetOutputRect()
{
	CRect rect;
	GetDlgItem(IDC_OUTPUT)->GetWindowRect(rect);
	ScreenToClient(rect);
	rect.InflateRect(-2, -2);	// leave border alone
	return rect;
}

// functions to map Dr Dobbs API calls to CDC member functions
static void _MoveTo(CDC* pDC, int x, int y)
	{ pDC->MoveTo(x, y); }
static void _LineTo(CDC* pDC, int x, int y)
	{ pDC->LineTo(x, y); }

// Render a character in the DC passed in
// all drawing takes place here for maximum control and reuse
void CHwbDlg::RenderInstance(CDC* pDC, char ch, int iInstance,
		CRect rect, int inflate, BOOL bAttrib)
{
	// fill in background
	if (inflate)
		pDC->FillRect(rect, &m_brBack);

	if (iInstance < 0)
		return;	// nothing more to do

	rect.InflateRect(inflate, inflate);
	if (inflate != 0 && bAttrib)
		pDC->FrameRect(rect, &m_brGray);

	pDC->SaveDC();
	// draw scaled to fit with optional attributes
	CPen pen(PS_INSIDEFRAME, m_nLineWidth*2+1, m_colorCur);
	if (bAttrib)
		pDC->SelectObject(&pen);

	pDC->SetMapMode(MM_ANISOTROPIC);
	if (!rect.IsRectEmpty())
	{
		pDC->SetViewportExt(rect.Size());
		pDC->SetViewportOrg(rect.TopLeft());
	}

	CRect rect2 = ::CalcInstanceBoundingBox(ch, iInstance);
	pDC->SetWindowExt(rect2.Size());
	pDC->SetWindowOrg(rect2.TopLeft());

	::RegisterCallback(pDC, _MoveTo, _LineTo);
	::DisplayInstance(ch, iInstance);
	pDC->RestoreDC(-1);
}

void CHwbDlg::OnPaint()
{
	CPaintDC dc(this);
	RenderInstance(&dc, m_chCur, m_iInstanceCur, GetOutputRect(), -10, TRUE);
}

void CHwbDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar*)
{
	static int deltas[] = { -1, 1, -1, 1, 0, 0, -1000, 1000, 0 };
	int iSel = nPos;
	if (nSBCode != SB_THUMBTRACK && nSBCode != SB_THUMBPOSITION)
		iSel = m_iInstanceCur + deltas[nSBCode];

	if (iSel < 0)
		iSel = 0;
	else if (iSel >= m_nInstanceCount)
		iSel = m_nInstanceCount-1;
	m_imageBox.SetCurSel(iSel);
	OnInstanceSelect();
}

// menu prompt string tracking
void CHwbDlg::OnMenuSelect(UINT nItemID, UINT nFlags, HMENU)
{
	CString strPrompt;
	if (nItemID != 0 &&
	  !(nFlags & (MF_SEPARATOR|MF_POPUP|MF_MENUBREAK|MF_MENUBARBREAK)))
		strPrompt.LoadString(nItemID);
	SetDlgItemText(IDC_STATUS, strPrompt);
}


// Line width: handle with one function for efficiency for a range of commands
BOOL CHwbDlg::OnCommand(WPARAM wParam, LPARAM lParam)
{
	int nWidth = LOWORD(wParam) - IDM_LINE_WIDTH_BASE;
	if (nWidth < 0 || nWidth >= MAX_LINE_WIDTH)
		return CModalDialog::OnCommand(wParam, lParam);	// not line width

	m_nLineWidth = nWidth;
	// update the menu
	for (int i = 0; i < MAX_LINE_WIDTH; i++)
		GetMenu()->CheckMenuItem(IDM_LINE_WIDTH_BASE + i,
			(i == m_nLineWidth) ? MF_CHECKED : MF_UNCHECKED);
	InvalidateRect(GetOutputRect(), FALSE);		// redraw
	return TRUE;
}

void CHwbDlg::OnLineColor()
{
	CColorDialog dlg(m_colorCur);
	if (dlg.DoModal() != IDOK)
		return;
	m_colorCur = dlg.GetColor();
	InvalidateRect(GetOutputRect(), FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// Diagnostics

#ifdef _DEBUG
void CHwbDlg::OnShowStrokes()
{
	// debug support to print out strokes

	lpList l;
	lpVHPoint p;

	l = GetInstanceDataEx(m_chCur, m_iInstanceCur);

	afxDump << "Stroke dump for " << CString(m_chCur) <<
 			" instance " << m_iInstanceCur << "\n";

	for (int i = 0; i < l->num_items; i++)
	{
		p = (lpVHPoint)l->items[i];
		afxDump << "Stroke (" << i << "): " << p->v << 
				", " << p->h << "\n";
	}
	afxDump << "\n";
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CImageBox - image self draw dropdown list

void CImageBox::MeasureItem(LPMEASUREITEMSTRUCT lpMIS)
{
	lpMIS->itemHeight = 32;
}

void CImageBox::DrawItem(LPDRAWITEMSTRUCT lpDIS)
{
	// draw it
	CDC* pDC = CDC::FromHandle(lpDIS->hDC);

	char ch = m_pParent->m_chCur;
	int i = lpDIS->itemID;
	if (this == &m_pParent->m_charBox && i >= 0)
	{
		ch = (char)lpDIS->itemData;
		i = 0;
	}
	m_pParent->RenderInstance(pDC, ch, i, &lpDIS->rcItem, -2, FALSE);

	if (lpDIS->itemState & ODS_SELECTED)
	{
		CBrush br(RGB(255, 0, 0));
		pDC->FrameRect(&lpDIS->rcItem, &br);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CHwbApp

BOOL CHwbApp::InitInstance()
{
	// Creates a dialog and runs it
	CHwbDlg mainDlg;
	m_pMainWnd = &mainDlg; // dialog is the main window
	mainDlg.DoModal();

	// that's all, quit app
	::PostQuitMessage(0);
	return TRUE;
}

CHwbApp theApp; // declare a single object of our application class
