#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <setjmp.h>
#include <malloc.h>

#include "hwx.h"

#define FILE_SIGNATURE 0x2121
#define MAX_CHARS 256

static jmp_buf main_loop;
static lpList  CharData[MAX_CHARS];
static long num_bytes = 0;


static INT16   ReadWordFromFile(FILE *data_file);
static void    ParseDataFormat(FILE *data_file);
static void    FreeMemStructures(void);
static void    FatalError(char* msg);

static void (*pf_MoveTo)(HDISPLAY,INT16,INT16);
static void (*pf_LineTo)(HDISPLAY,INT16,INT16);
static HDISPLAY theDisplay;


/****************************************************************/
void LoadHWXData(const char* file_name)
{
    FILE *data_file;

    data_file = fopen(file_name,"rb");
    if (!data_file) 
	{
		char szBuffer[255];
		sprintf(szBuffer, "File %s not found.", file_name);
		FatalError(szBuffer);
	}
    FreeMemStructures();
    ParseDataFormat(data_file);
    fclose(data_file);
}

/****************************************************************/
void UnloadHWXData(void)
{
    FreeMemStructures();
}

/****************************************************************/
int GetInstanceCount(int ch)
{
    if(CharData[ch]) return CharData[ch]->num_items;
    else return 0;
}

void RegisterCallback(
       HDISPLAY display_entity,
       void (*pf_move_to)(HDISPLAY,INT16,INT16),
       void (*pf_line_to)(HDISPLAY,INT16,INT16))
{
    pf_MoveTo = pf_move_to;
    pf_LineTo = pf_line_to;
	theDisplay = display_entity;
}

/****************************************************************/
lpList GetInstanceDataEx(int ch,int inst)
{
    lpList instance_data;
    if(!CharData[ch]) return 0;
    instance_data = (lpList)CharData[ch]->items[inst];
    if(!instance_data) return 0;
    /* THe instance_data variable now points to List of strokes, which 
     * themselves point to a List of points.
     * The number of list elements is in instance_data->num_items.
     */
    return instance_data;
}

void FatalError(char * msg)
{
	MessageBox(NULL, msg, "Fatal Error", MB_OK);
	exit(-1);
}

/***************************************************************
|    OTHER SUPPORTING ROUTINES
\****************************************************************/
static void FreeMemStructures(void)
{
    INT16 i,j,k;
    for (i = 0; i < MAX_CHARS; i++) if (CharData[i]) 
    {
        for (j = 0; j < CharData[i]->num_items; j++) 
	{
            lpList char_instance = (lpList)CharData[i]->items[j];
            for (k = 0; k < char_instance->num_items; k++)
                _ffree(char_instance->items[k]);
            _ffree(char_instance);
        }
        _ffree(CharData[i]);
        CharData[i] = NULL;
    }
}

/****************************************************************/
static INT16  ReadWordFromFile(FILE *data_file)
{
    char s[2];
    s[0] = s[1] = 0;
    if(fread(&s, sizeof(char), 2, data_file) != 2)
	longjmp( main_loop, -1 );
    num_bytes += 2;
    return (INT16)( ((BYTE)s[1]) | ((WORD)((BYTE)s[0])<<8) );
}

/****************************************************************/
static void ReadHeader(FILE *data_file)
{
    INT16 signature,header_size,version_major,version_minor;
    signature = ReadWordFromFile(data_file);
    if(signature != FILE_SIGNATURE) FatalError("***Header format error!***");
    header_size = ReadWordFromFile(data_file); /*number of bytes should be even */
    if(header_size & 01) FatalError("***Header format error #2!***");
    header_size >>= 1;  /* number of words */
    header_size--;
    version_major = ReadWordFromFile(data_file); header_size--;
    version_minor = ReadWordFromFile(data_file); header_size--;    
    while(header_size--) ReadWordFromFile(data_file);
    /*the following line serves only to get rid of a compiler warning */
    version_major++; version_minor++; 
}

/****************************************************************/
static void ParseDataFormat(FILE *data_file)
{
    VHPoint Pt; 
    lpList   char_instance;
    lpList   stroke;
    INT16    i,j,k,num_strokes,num_pts,which;
    INT16    char_code,num_instances;

    if( setjmp( main_loop )==0 ) 
    {
	ReadHeader(data_file);
	while (1)
        {
        char_code = ReadWordFromFile(data_file);
        num_instances = ReadWordFromFile(data_file);

        which = (char_code < MAX_CHARS) ? char_code : 1;
        CharData[which] = (lpList)_fmalloc(sizeof(INT16) 
			    + (num_instances * sizeof(long)));
        CharData[which]->num_items = num_instances;
        for (i = 0; i < num_instances; i++) 
	{
            num_strokes = ReadWordFromFile(data_file);

            char_instance = (lpList)_fmalloc(sizeof(INT16) + 
				(num_strokes * sizeof(long)));

            char_instance->num_items = num_strokes;

            for (j = 0; j < num_strokes; j++) 
	    {
                num_pts = ReadWordFromFile(data_file);
		Pt.h = ReadWordFromFile(data_file);
		Pt.v = ReadWordFromFile(data_file);

                stroke = (lpList)_fmalloc(sizeof(INT16) + num_pts * sizeof(long));
                stroke->num_items = num_pts;
                stroke->items[0] = *(GenericT*)&Pt;
                for (k = 1; k < num_pts; k++) 
		{
		    INT16 s = ReadWordFromFile(data_file);
		    LPSTR ps = (LPSTR )&s;

                    Pt.h += ps[1]; Pt.v += ps[0];
                    stroke->items[k] = *(GenericT*)&Pt;
                } /* for loop (k) */
                char_instance->items[j] = stroke;

            } /* for loop (j) */
            CharData[which]->items[i] = char_instance;
           }  /* for loop (i) */                  
         } /*while loop */
       } /*if*/
       else if (!feof(data_file))
       {
	    char szBuffer[255];
	    sprintf(szBuffer, "Error reading file at offset %ld", num_bytes);
	    FatalError(szBuffer);
       }
}

void DisplayInstance(int char_code,int instance_num)
{
    lpList char_instance;
    INT16 i,j;

    char_instance = CharData[char_code]->items[instance_num];
    
    for (i = 0; i < char_instance->num_items; i++) 
    {
        lpList stroke =   char_instance->items[i];

        for (j = 0; j < stroke->num_items; j++) 
	{
            VHPoint p = *(lpVHPoint)&stroke->items[j];
	    if (j == 0)
		(*pf_MoveTo)(theDisplay,p.h,p.v);
	    (*pf_LineTo)(theDisplay,p.h,p.v);
        }
    }
}

/****************************************************************/
#define MIN_VALUE (-0x7FFF)
#define MAX_VALUE ( 0x7FFF)

RECT CalcInstanceBoundingBox(int char_code, int instance_num)
{
    INT16 i,j;
    INT16 x_min,x_max,y_min,y_max;
	RECT r;
	lpList char_instance = GetInstanceDataEx(char_code, instance_num);

     x_min = y_min = MAX_VALUE;
     x_max = y_max = MIN_VALUE;

    /*------for a given char, find the maximum and minimum -------*/
    for (i = 0; i < char_instance->num_items; i++) 
    {
        lpList    stroke =   char_instance->items[i];

        for (j = 0; j < stroke->num_items; j++) 
	{
            VHPoint p = *(lpVHPoint)&stroke->items[j];
            if (p.h > x_max) x_max = p.h;
            if (p.h < x_min) x_min = p.h;
            if (p.v > y_max) y_max = p.v;
            if (p.v < y_min) y_min = p.v;
        }
    }
    r.left  = x_min;
    r.top   = y_min; 
    r.right = x_max + 2;
    r.bottom = y_max + 2;
	return r;
}
