#include <owl.h>
#include <dir.h>
#include <bwcc.h>
#include <windowsx.h>
#include <filedial.h>
#include <inputdial.h>


#include "ids.h"
#include "color.h"
#include "printer.h"
#include "gridview.h"
#include "viewwind.h"
#include "hwxbrowz.h"
#include "h_file.h"



_CLASSDEF( TMainWindow )
_CLASSDEF( TMainApp )


//
//  Constructor of Application's MainWindow.
//  Initialize various variables...
//
TMainWindow::TMainWindow( PTWindowsObject AParent, LPSTR AName, PTModule AModule )
        : TDialog(  AParent, AName, AModule )
{
    fDataLoaded= FALSE;

    DrawData.cur_char = 'A';
    DrawData.cur_inst =  0;
    DrawData.rgbBackGroundColor = GetSysColor( COLOR_WINDOW );
    DrawData.rgbLineColor       = GetSysColor( COLOR_WINDOWTEXT );
    DrawData.nLineThickness     = 1;
    DrawData.nScaleFactor       = NORM_SCALE;


    GridWindow     = new TGridWindow( this, ID_GRIDWINDOW , &DrawData );
    ViewWindow     = new TViewWindow( this, ID_VIEWWINDOW , &DrawData );
    Printer        = new TPrinter;

    ScaleScrollBar = new TScrollBar( this, ID_SCROLLSCALE );
    ScaleText      = new TStatic   ( this, ID_SCALETEXT,-1);
}



//
//  Destructor of App's MainWindow
//  CleanUp Code...
//
TMainWindow::~TMainWindow()
{
    if ( Printer )
        delete Printer;
}



//
//  Main Window's Class Name
//
LPSTR TMainWindow::GetClassName()
{
    return "bordlg_MainDialog";
}



//
//  Use the Borland Custom Dialog Class and
//  indicate the Application's Icon.
//
void TMainWindow::GetWindowClass( WNDCLASS& AWndClass )
{
    TDialog::GetWindowClass( AWndClass );
    AWndClass.lpfnWndProc = BWCCDefDlgProc;
    AWndClass.hIcon = LoadIcon( GetApplication()->hInstance,
                                "ApplicationIcon" );
}



//
//  Function called as a result of the WM_INITDIALOG message.
//  Center Dialog on Screen...
//
void TMainWindow::SetupWindow()
{
    RECT  rc;

    TDialog::SetupWindow();
    GetWindowRect( HWindow, &rc );
    OffsetRect( &rc, -rc.left, -rc.top );
    MoveWindow( HWindow,
                (( GetSystemMetrics( SM_CXSCREEN ) -
                   rc.right ) / 2 + 4 ) & ~7,
                 ( GetSystemMetrics( SM_CYSCREEN ) -
                   rc.bottom ) / 2,
                   rc.right, rc.bottom, 0 );

    ScaleScrollBar->SetRange( 1, 10 );
    ScaleScrollBar->SetPosition( 5 );


    EnableScaleScrollBar( FALSE );
    ScaleText->SetText( "&Scale:  50%" );

    SetFocus( GridWindow->HWindow );
}





void TMainWindow::EnableScaleScrollBar( BOOL fFlag )
{
    EnableWindow( ScaleScrollBar->HWindow, fFlag );
    EnableWindow( ScaleText->HWindow, fFlag );
}




//
//  Function responds to FILE|READ Menu Option...
//  Prompts user for FileName and Read Data...
//
void TMainWindow::CMFileRead ( RTMessage )
{
    strcpy( lpszFileName , "*.dat" );
    GetApplication()->ExecDialog( new TFileDialog( this , 
                                       SD_FILEOPEN , lpszFileName ));
    SetWait();
    if ( file_LoadHWXData( lpszFileName ))
    {
        fDataLoaded = TRUE;
        GridWindow->UpdateView();
        ViewWindow->UpdateView();
    }
    ReleaseWait();
}



//
//  Responds to the Print Request...
//
void TMainWindow::CMPrint(RTMessage)
{
    PTPrintCharInst Printout = 0;

    if ( Printer )
    {
        Printout = new TPrintCharInst( "Char Instance",
                                        DrawData.cur_char,
                                        DrawData.cur_inst,
                                        DrawData.nLineThickness,
                                        DrawData.nScaleFactor );
        if ( Printout )
        {
            Printout->SetBanding( TRUE );
            Printer->Print(this, Printout);
            delete Printout;
        }
    }
}



//
//  Allows user to Select/Configure Printer
//
void TMainWindow::CMPrinterSetup(RTMessage)
{
    if ( Printer )
        Printer->Setup(this);
}



//
//  Request to Exit Menu Choice - Terminates App...
//
void TMainWindow::CMExit( RTMessage )
{
    PostMessage( HWindow, WM_SYSCOMMAND, SC_CLOSE, 0 );    
}



//
//  Bring up AboutBox ...
//
void TMainWindow::CMAbout( RTMessage )
{
	GetApplication()->ExecDialog( new TDialog( this , "ABOUTBOX" ));
}



//
//
//
void TMainWindow::UpdateSubViews( void )
{
    GridWindow->UpdateView();
    ViewWindow->UpdateView();
}



//
//  Allows user to select Foreground/Line Color.
//
void TMainWindow::IDForColor( RTMessage )
{
    PTColorDialog ColorDialog = new TColorDialog( this ,
                                                  DrawData.rgbLineColor );
    if ( ColorDialog )
    {
        delete ColorDialog;
        UpdateSubViews();
    }
}



//
// Allows user to select Background Color..
//
void TMainWindow::IDBackColor( RTMessage )
{
    PTColorDialog ColorDialog = new TColorDialog( this ,
                                                  DrawData.rgbBackGroundColor ,
                                                  TRUE );
    if ( ColorDialog )
    {
        delete ColorDialog;
        UpdateSubViews();
    }
}



//
// Responds to Line Thickness Selection.
//
void TMainWindow::IDLineRad1( RTMessage )
{
    DrawData.nLineThickness = 1;
    UpdateSubViews();
}


//
//  Responds to Line Thickness Selection.
//
void TMainWindow::IDLineRad2( RTMessage )
{
    DrawData.nLineThickness = 5;
    UpdateSubViews();
}



//
//  Responds to Line Thickness Selection.
//
void TMainWindow::IDLineRad3( RTMessage )
{
    DrawData.nLineThickness = 9;
    UpdateSubViews();
}



//
//  Responds to Line Thickness Selection.
//
void TMainWindow::IDLineRad4( RTMessage )
{
    DrawData.nLineThickness = 13;
    UpdateSubViews();
}



//
//  Allows user to specify Scale for GridDisplay...
//
void TMainWindow::IDScrollScale( RTMessage )
{
    int nPos = ScaleScrollBar->GetPosition();
    if ( nPos != DrawData.nScaleFactor )
    {
        char buff[80];

        wsprintf( buff, "&Scale: %d%%", nPos*10 );
        
        DrawData.nScaleFactor = MAX_SCALE + MIN_SCALE - nPos;
        ScaleText->SetText( buff );
        GridWindow->UpdateView();
    }
}



//
//  Sent by GridWindow to MainWindow which then informs
//  the ViewWindow of the change !
//
void TMainWindow::SetCurChar( int ch, int inst )
{
    if ( DrawData.cur_char != ch  ||  DrawData.cur_inst != inst )
    {
        DrawData.cur_char = ch;
        DrawData.cur_inst = inst;
        ViewWindow->UpdateView();   // Allows View to Refresh
    }
}




//
//  Saves current cursor and Loads HourGlass !
//
void  TMainWindow::SetWait( void )
{
    hCurSave = SetCursor( LoadCursor( NULL , IDC_WAIT ));
}



//
//  Restores prior cursor...
//
void  TMainWindow::ReleaseWait( void )
{
    SetCursor( hCurSave );
}



//
//  Constructor of Main Application Class...
//
TMainApp::TMainApp( LPSTR AName, HINSTANCE hInstance, 
                    HINSTANCE hPrevInstance, LPSTR lpCmd, int nCmdShow ):
TApplication( AName, hInstance, hPrevInstance, lpCmd, nCmdShow )
{
}



//
//  Specifying the Application's Main Window...
//
void TMainApp::InitMainWindow()
{
    BWCCGetVersion();   // Force Implicit Loading of BWCC.DLL !
    MainWindow = new TMainWindow( NULL,  "MainDialog" );
}



//
//  Create Application Class and run !
//
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpCmd, int nCmdShow )
{
    TMainApp  MyApp( "HWX Browser", hInstance, 
                      hPrevInstance, lpCmd, nCmdShow );
    MyApp.Run();
    return( MyApp.Status );
}
