/*-------------------------------------------------------------*\
 | H_File.C -- File i/o support routines for Charview.EXE.     |
\*-------------------------------------------------------------*/

#include <Windows.H>
#include <Stdio.H>

#include "Chv.H"

PRIVATE void  file_FreeMemStructures(void);
PRIVATE INT16 file_ReadWordFromFile(FILE *data_file);
PRIVATE void  file_ParseDataFormat(FILE *data_file);
PRIVATE void  file_ReadHeader(FILE *data_file);

extern   LPLIST   CharData[];
extern   char     chErrBuf[];
PRIVATE  CATCHBUF catchbuf;
PRIVATE  LONG     num_bytes;

/****************************************************************/
PUBLIC void file_LoadHWXData(LPCSTR file_name)
{
    FILE *data_file;
    data_file = fopen(file_name,(LPCSTR)"rb");
    if (!data_file) 
        {
        wsprintf (chErrBuf,"***File %s not found!",(LPSTR)file_name);
        FatalError(chErrBuf);
        }
    file_FreeMemStructures();
    file_ParseDataFormat(data_file);
    fclose(data_file);
}
/****************************************************************/
PUBLIC void file_UnloadHWXData(void)
{
    file_FreeMemStructures();
}
/****************************************************************/
PRIVATE void file_ParseDataFormat(FILE *data_file)
{
    POINT   Pt;
    LPLIST  char_instance;
    LPLIST  stroke;
    INT16   i,j,k,num_strokes,num_pts,which;
    INT16   char_code,num_instances;

    if( Catch ((INT16 FAR *) catchbuf)==0 )
        {
        file_ReadHeader(data_file);
        while (1)
            {
            char_code = file_ReadWordFromFile(data_file);
            num_instances = file_ReadWordFromFile(data_file);

            which = (char_code < MAX_CHARS) ? char_code : 1;
            CharData[which] = (LPLIST)mem_Alloc(sizeof(INT16)
                    + (num_instances * sizeof(long)));
            if(!CharData[which])
                mem_AllocFailed(sizeof(INT16)
                    + (num_instances * sizeof(long)));
            CharData[which]->num_items = num_instances;
            for (i = 0; i < num_instances; i++)
                {
                num_strokes = file_ReadWordFromFile(data_file);

                char_instance = (LPLIST)mem_Alloc(sizeof(INT16) +
                        (num_strokes * sizeof(long)));
                if(!char_instance)
                    {
                    mem_AllocFailed(sizeof(INT16)
                        + (num_strokes * sizeof(long)));
                    }
                char_instance->num_items = num_strokes;

                for (j = 0; j < num_strokes; j++)
                    {
                    num_pts = file_ReadWordFromFile(data_file);
                    Pt.x = file_ReadWordFromFile(data_file);
                    Pt.y = file_ReadWordFromFile(data_file);

                    stroke = (LPLIST)mem_Alloc(sizeof(INT16) +
                                       num_pts * sizeof(long));
                    if(!stroke)
                        mem_AllocFailed(sizeof(INT16) +
                                        num_pts * sizeof(long));

                    stroke->num_items = num_pts;
                    stroke->items[0] = *(void**)&Pt;
                    for (k = 1; k < num_pts; k++)
                        {
                        INT16 s = file_ReadWordFromFile(data_file);
                        LPSTR ps = (LPSTR )&s;

                        Pt.x += ps[1]; Pt.y += ps[0];
                        stroke->items[k] = *(void**)&Pt;
                        } /* for loop (k) */

                    char_instance->items[j] = (void *)stroke;

                    } /* for loop (j) */
                CharData[which]->items[i] = (void *)char_instance;
                }  /* for loop (i) */
            } /*while loop */
        } /*if*/
    else if (!feof(data_file))
        {
        wsprintf (chErrBuf,"***Err reading file at loc %ld!",num_bytes);
        FatalError(chErrBuf);
        }
}
/****************************************************************/
PRIVATE void file_ReadHeader(FILE *data_file)
{
    INT16 signature,header_size,version_major,version_minor;
    signature = file_ReadWordFromFile(data_file);
    if(signature != FILE_SIGNATURE) FatalError("***Header format error!***");
    header_size = file_ReadWordFromFile(data_file); //num bytes should be even
    if(header_size & 01) FatalError("***Header format error #2!***");
    header_size >>= 1;  // number of words
    header_size--;
    version_major = file_ReadWordFromFile(data_file); header_size--;
    version_minor = file_ReadWordFromFile(data_file); header_size--;
    while(header_size--) file_ReadWordFromFile(data_file);
    //the following line serves only to get rid of a compiler warning
    version_major++; version_minor++; 
}
/****************************************************************/
PRIVATE INT16  file_ReadWordFromFile(FILE *data_file)
{
    char s[2];
    s[0] = s[1] = 0;
    if(fread(&s, sizeof(char), 2, data_file) != 2)
        Throw( (INT16 FAR *) catchbuf, -1);
    num_bytes += 2;
    return (INT16)( ((BYTE)s[1]) | ((WORD)((BYTE)s[0])<<8) );
}
/****************************************************************/
PUBLIC void FatalError(char* msg)
{
    MessageBox (NULL, msg, "CharView", MB_OK);
}
/****************************************************************/
PRIVATE void file_FreeMemStructures(void)
    {
    INT16 i,j,k;

    for (i = 0; i < MAX_CHARS; i++) if (CharData[i]) 
        {
        for (j = 0; j < CharData[i]->num_items; j++) 
            {
            LPLIST char_instance =(LPLIST)CharData[i]->items[j];
            for (k = 0; k < char_instance->num_items; k++)
                mem_Free(char_instance->items[k]);
            mem_Free(char_instance);
            }
        mem_Free(CharData[i]);

        CharData[i] = NULL;
        }
    }
