/****************************************************************
| H_MEM.C  -- memory allocation module
+----------------------------------------------------------------
| The purpose of this is to provide a layer between the application
| and the memory allocation routines in the environment.
| The original Macintosh code did not use this layer, but called
| the standard library directly. In order to facilitate the
| move to MS-Windows, this layer was added.
+****************************************************************/


#include "h_config.h"

#define PUBLIC
#define PRIVATE static

#ifdef TURBO_C
#include <stdio.h>
#include <alloc.h>
#endif

#ifdef MSOFT_C7
#include <stdio.h>
#include <malloc.h>

#include <process.h>
#endif

#include <windows.h>
#include "h_mem.h"


extern char chErrBuf[];
void  FatalError(char* msg);

/****************************************************************/
PRIVATE long num_operations = 0;
PRIVATE long bytes_allocd = 0;


/****************************************************************/
PUBLIC void *mem_Realloc(void *v,size_t num_bytes)
{
#ifdef MS_DOS
#ifdef MSOFT_C7_MEM_DBG
    size_t n = _msize(v);
    bytes_allocd -= n;
#endif
    num_operations++;
    return realloc(v,num_bytes);
#endif
}
/****************************************************************/
PUBLIC void mem_Free(void *v)
{
#ifdef MS_DOS
#ifdef MSOFT_C7_MEM_DBG
    size_t n = _msize(v);
    bytes_allocd -= n;
#endif
    num_operations++;

    if ( v )
       free(v);
#endif
}
/****************************************************************/
PUBLIC void * mem_Alloc(size_t num_bytes)
{
#ifdef MS_DOS
    void *v = malloc(num_bytes);
    bytes_allocd += num_bytes;
    num_operations++;
    return v;
#endif
}
/****************************************************************/
/* DELETE
PUBLIC void * mem_CAlloc(size_t num_items, size_t item_size)
{
#ifdef MS_DOS
    void *v = calloc(num_items, item_size);
    long num_bytes = (long)(num_items * item_size);
    bytes_allocd += num_bytes;
    num_operations++;
    return v;
#endif
}
*/

/****************************************************************/
PUBLIC void mem_AllocFailed(size_t num_bytes)
{
#ifdef MS_DOS
    wsprintf (chErrBuf, "***Mem alloc #%ld failed on %ld bytes***",
        (long)num_operations,(long)num_bytes);

    FatalError (chErrBuf);
#endif
}
