#include <owl.h>
#include <mem.h>
#include <stdio.h>

#include "h_file.h"
#include "showdraw.h"


static int DrawCharGen( HDC hdc, RECT &boundRect , LPLIST lpCharInstance,
                        BOOL bDrawLine , int nScale );

extern LPLIST CharData[];



//
//  Computes the Bounding Rectangle of a character
//
void GetCharRect( LPLIST lpInstance , RECT &rect )
{
    rect.left   = USHRT_MAX/2;
    rect.top    = USHRT_MAX/2;
    rect.right  = 0;
    rect.bottom = 0;

    for ( int i = 0; i < lpInstance->num_items; i++ )
    {
        LPLIST lpStroke = (LPLIST)(lpInstance->items[i]);
        LPPOINT lppt = (LPPOINT)(&lpStroke->items[0]);

        for ( int j = 0; j < lpStroke->num_items; j++ )
        {
            rect.left  = min( rect.left , lppt->x );
            rect.top   = min( rect.top  , lppt->y );
            rect.right = max( rect.right, lppt->x );
            rect.bottom= max( rect.bottom,lppt->y );
            lppt++;
        }
    }
}



//
//  Draws the specified instance of a character within
//  the specified rectangle using the specified scale.
//
int DrawCharLines( HDC hdc,
                   RECT &boundRect,
                   int ch,
                   int inst_index,
                   int nScale )
{
        return DrawCharGen( hdc ,
                            boundRect ,
                            GetCharInstanceData( ch , inst_index ) ,
                            TRUE ,
                            nScale );
}



//
//  Helper function to compute the delta between the sides
//  of a rectangle.   Returns the delta in a POINT Structure.
//
POINT GetRectExt( RECT r )
{
    static POINT ptTemp;

    ptTemp.x = r.right - r.left;
    ptTemp.y = r.bottom - r.top;

    return ptTemp;
}



//
//  Draws the specified instance of a character using pixels ( points )
//  and not strokes( lines ).
//
int DrawCharPoints( HDC hdc, RECT &boundRect,
                    int ch, int cur_inst, int nScale )
{
    return DrawCharGen( hdc , boundRect , GetCharInstanceData( ch , cur_inst ),
                        FALSE, nScale );
}



//
//  Helper function doing the actual drawing of characters.
//  Called by the both DrawCharLines and DrawCharPoints.
//
static int DrawCharGen( HDC hdc,
                        RECT &boundRect,
                        LPLIST lpCharInstance,
                        BOOL bDrawLines, 
                        int nScale )       
{
    if ( !lpCharInstance )
        return 0; 

    int nSavedMode = GetMapMode( hdc );

    RECT charRect;

    GetCharRect( lpCharInstance , charRect );

    POINT CharRectExt  = GetRectExt( charRect );
    POINT BoundRectExt = GetRectExt( boundRect );

    SetMapMode( hdc , MM_ISOTROPIC );

    SetWindowExt( hdc ,
                ((CharRectExt.x  ) * nScale ) / AveScale ,
                ((CharRectExt.y  ) * nScale ) / AveScale );

    SetViewportExt( hdc, BoundRectExt.x, BoundRectExt.y );
    SetViewportOrg( hdc, boundRect.left, boundRect.top );

    SetWindowOrg( hdc, charRect.left, charRect.top );

    for( int i=0; i < lpCharInstance->num_items; i++ )
    {
        LPLIST  lpStroke = (LPLIST)(lpCharInstance->items[i]);
        LPPOINT lppt     = GetFirstPoint( lpStroke );

        if (bDrawLines)
        {
            Polyline( hdc, lppt , lpStroke->num_items );
        }
        else
        {
            for ( int j = 0; j < lpStroke->num_items; j++ )
            {
                SetPixel( hdc , lppt->x, lppt->y , GetSysColor( COLOR_WINDOWTEXT ));
                lppt++;
            }
        }
    }
    SetMapMode( hdc, nSavedMode );
    return 1;
}


