/*-------------------------------------------------------------*\
 | VIEW.CPP  -- TCharViewWindow class member functions         |
\*-------------------------------------------------------------*/
#define WIN31
#define STRICT

#include <Owl.H>
#include <WindowsX.h>

extern "C" {
#include "Chv.H"
};

extern BYTE   chCurrent;
extern BYTE   chErrBuf[];
extern int    inCurrent;
extern LPLIST CharData[];

#include "Classes.H"

/*----------------Handwriting Parsing Routine------------------*/
PRIVATE void hw_CalcInstanceBoundingBox(LPLIST,LPRECT);

const int MIN_VALUE = -0x7FFF;
const int MAX_VALUE =  0x7FFF;

/*-------------------------------------------------------------*\
 |                TCharViewWindow Class Member.                |
\*-------------------------------------------------------------*/
TCharViewWindow::TCharViewWindow (PTWindowsObject pwParent,
               LPSTR lpszTitle, PTModule pmModule, int iScaling,
               int iRows, int iColumns, int iInstStart,
               int iInstEnd, int iListBox)
           :TWindow (pwParent, lpszTitle, pmModule)
    {
    /*  Window specific initialization goes here.  */
    Attr.Style    = WS_VISIBLE | WS_BORDER | WS_CHILD;

    hpenLetter    = CreatePen (PS_SOLID, 0, RGB (255, 0, 0));
    hpenTextCell  = GetStockPen (BLACK_PEN);
    hpenBoundBox  = CreatePen (PS_DASH, 0, RGB (0, 0, 255) );
    hpenCurrent   = CreatePen (PS_SOLID, 2, RGB (0, 255, 0) );

    chCurrent     = 'A';
    cRows         = iRows;
    cColumns      = iColumns;
    instFirst     = iInstStart;
    instLast      = iInstEnd;
    instCurrent   = 0;
    cyListBox     = iListBox;

    bDrawComplete = (iInstStart == iInstEnd) ? TRUE : FALSE;

    SetScalingFactor (iScaling);
    }

/*-------------------------------------------------------------*\
 |                TCharViewWindow Class Member.                |
\*-------------------------------------------------------------*/
TCharViewWindow::~TCharViewWindow()
    {
    DeletePen (hpenLetter);
    DeletePen (hpenBoundBox);
    DeletePen (hpenCurrent);
    }

/*-------------------------------------------------------------*\
 |                TCharViewWindow Class Member.                |
\*-------------------------------------------------------------*/
void
TCharViewWindow::SetCurrentChar (BYTE chNew)
    {
    chCurrent = chNew;
    InvalidateRect (HWindow, NULL, TRUE);
    }

/*-------------------------------------------------------------*\
 |                TCharViewWindow Class Member.                |
\*-------------------------------------------------------------*/
void
TCharViewWindow::SetScalingFactor (int iScaling)
    {
    int cxScreen, cyScreen;

    cxScreen = GetSystemMetrics (SM_CXFULLSCREEN);
    cyScreen = GetSystemMetrics (SM_CYFULLSCREEN) -
               GetSystemMetrics (SM_CYMENU) -
               cyListBox;

    // Divide screen between two views of characters.
    if (bDrawComplete)
        {
        cxScreen = MulDiv (cxScreen, 1, 4);
        }
    else
        {
        cxScreen = MulDiv (cxScreen, 3, 4);
        }

    // Adjust for scaling factor.
    cxScreen = MulDiv (cxScreen, iScaling, 100);
    cyScreen = MulDiv (cyScreen, iScaling, 100);

    cxMargin = GetSystemMetrics (SM_CXICON) / 2;
    cyMargin = GetSystemMetrics (SM_CYICON) / 2;

    if (bDrawComplete)
        {
        cxCell = cxScreen - (cxMargin * 2);
        cyCell = cyScreen / 2;
        }
    else
        {
        cxCell = (cxScreen - (cxMargin * 2)) / cColumns;
        cyCell = (cyScreen - (cyMargin * 2)) / cRows   ;
        }

    InvalidateRect (HWindow, NULL, TRUE);
    }

/*-------------------------------------------------------------*\
 |                TCharViewWindow Class Member.                |
\*-------------------------------------------------------------*/
LPSTR
TCharViewWindow::GetClassName ()
    {
    return "CHARVIEW:VIEWER";
    }

/*-------------------------------------------------------------*\
 |                TCharViewWindow Class Member.                |
\*-------------------------------------------------------------*/
void
TCharViewWindow::GetWindowClass (WNDCLASS& wc)
    {
    TWindow::GetWindowClass (wc);
    wc.hIcon        = NULL;
    wc.hCursor      = LoadCursor (wc.hInstance, "hand");
    }

/*-------------------------------------------------------------*\
 |                TCharViewWindow Class Member.                |
\*-------------------------------------------------------------*/
void
TCharViewWindow::WMLButtonDown(TMessage& Msg)
    {
    HWND         hwnd;
    int          iCount;
    POINT        ptMouse;
    PTMainWindow ptParent;

    if (!bDrawComplete)
        {
        ptMouse.x = MOUSEX(Msg);
        ptMouse.y = MOUSEY(Msg);

        for (iCount = 0; iCount < MAX_INSTANCE; iCount++)
            {
            if (PtInRect (&rHitTest[iCount], ptMouse) )
                {
                if (iCount != instCurrent)
                    {
                    // Update our window object data.
                    InvalidateRect (HWindow,
                                    &rHitTest[iCount],
                                    TRUE);
                    InvalidateRect (HWindow,
                                    &rHitTest[instCurrent],
                                    TRUE);

                    instCurrent = iCount;

                    // Update sibling window object data.
                    ptParent = (PTMainWindow)Parent;
                    ptParent->ptLeft->instCurrent = iCount;

                    hwnd = ptParent->ptLeft->HWindow;
                    InvalidateRect (hwnd, NULL, TRUE);
                    }
                break;
                }
            }
        }

    }

/*-------------------------------------------------------------*\
 |                TCharViewWindow Class Member.                |
\*-------------------------------------------------------------*/
void
TCharViewWindow::Paint(HDC hdc, PAINTSTRUCT& )
    {
    INT16       cx, cy;
    INT16       iStroke, iPoint;
    INT16       iRow, iColumn;
    INT16       iInstance;
    LPLIST      lpInstance;
    RECT        rText;
    RECT        rBounding;
    SIZE        sizeVENew;
    SIZE        sizeVE;

    // Only draw if we've got data for current character.
    if (CharData[chCurrent] != (LPLIST)NULL)
        {
        for (iRow = 0; iRow < cRows; iRow++)
            {
            for (iColumn = 0; iColumn < cColumns; iColumn++)
                {
                SetMapMode (hdc, MM_TEXT);
                SetWindowOrgEx (hdc, 0, 0, NULL);
                SetViewportOrgEx (hdc, 0, 0, NULL);

                SelectPen (hdc, hpenTextCell);

                if (bDrawComplete)
                    {
                    // Calculate instance.
                    iInstance = instCurrent;

                    // Calculate cell bounding box.
                    rText.left   = cxMargin;
                    rText.top    = cyMargin;
                    rText.right  = cxCell + cxMargin;
                    rText.bottom = cyCell + cyMargin;
                    }
                else
                    {
                    // Calculate instance.
                    iInstance = instFirst + (iRow * cColumns) + iColumn;

                    // Calculate cell bounding box.
                    rText.left   = cxMargin + (iColumn * cxCell);
                    rText.top    = cyMargin + (iRow    * cyCell);
                    rText.right  = rText.left + cxCell ;
                    rText.bottom = rText.top  + cyCell ;

                    // Save hit-test rectangle for mouse interface.
                    rHitTest[iInstance] = rText;

                    // Draw text cell.
                    MoveTo (hdc, rText.left,  rText.top);
                    LineTo (hdc, rText.right, rText.top);
                    LineTo (hdc, rText.right, rText.bottom);
                    LineTo (hdc, rText.left,  rText.bottom);
                    LineTo (hdc, rText.left,  rText.top);

                    // Highlight current instance.
                    if (iInstance == instCurrent)
                        {
                        SelectPen (hdc, hpenCurrent);
                        MoveTo (hdc, rText.left+2,  rText.top+2);
                        LineTo (hdc, rText.right-1, rText.top+2);
                        LineTo (hdc, rText.right-1, rText.bottom-1);
                        LineTo (hdc, rText.left+2,  rText.bottom-1);
                        LineTo (hdc, rText.left+2,  rText.top+2);
                        }

                    // Adjust for margin inside cell bounding box.
                    rText.left   += cxMargin / cRows;
                    rText.top    += cyMargin / cColumns;
                    rText.right  -= cxMargin / cRows;
                    rText.bottom -= cyMargin / cColumns;
                    }

                // Calculate offset to current data.
                lpInstance = (LPLIST)CharData[chCurrent]->items[iInstance];

                // World coordinates - calculate bounding box.
                hw_CalcInstanceBoundingBox (lpInstance, &rBounding);
                cx = rBounding.right - rBounding.left;
                cy = rBounding.bottom - rBounding.top;

                // Set up GDI mapping.
                SetMapMode (hdc, MM_ISOTROPIC);
                SetWindowOrgEx (hdc, rBounding.left, rBounding.top, NULL);
                SetViewportOrgEx (hdc, rText.left, rText.top, NULL);
                SetWindowExtEx (hdc, cx, cy, NULL);

                sizeVE.cx = rText.right - rText.left;
                sizeVE.cy = rText.bottom - rText.top;
                SetViewportExtEx (hdc, sizeVE.cx,
                                          sizeVE.cy, NULL);

                // Center drawing in text box.
                GetViewportExtEx (hdc, &sizeVENew);
                sizeVENew.cx = (sizeVENew.cx == sizeVE.cx ) ?
                                0 : (sizeVE.cx - sizeVENew.cx) /2;
                sizeVENew.cy = (sizeVENew.cy == sizeVE.cy ) ?
                                0 : (sizeVE.cy - sizeVENew.cy) /2;

                OffsetViewportOrgEx (hdc,sizeVENew.cx,
                                        sizeVENew.cy,NULL);

                // Draw bounding rectangle.
                SelectPen (hdc, hpenBoundBox);
                MoveTo (hdc, rBounding.left,  rBounding.top);
                LineTo (hdc, rBounding.right, rBounding.top);
                LineTo (hdc, rBounding.right, rBounding.bottom);
                LineTo (hdc, rBounding.left,  rBounding.bottom);
                LineTo (hdc, rBounding.left,  rBounding.top);

                // Stroke out character.
                SelectPen (hdc, hpenLetter);
                for (iStroke = 0;
                     iStroke < lpInstance->num_items;
                     iStroke++)
                    {
                    LPPOINT lppt;
                    LPLIST  lpStroke = (LPLIST)lpInstance->items[iStroke];

                    lppt = (LPPOINT)(&lpStroke->items[0]);

                    if (bDrawComplete)
                        {
                        Polyline (hdc, lppt, lpStroke->num_items);
                        }
                    else
                        {
                        for (iPoint = 0;
                             iPoint < lpStroke->num_items;
                             iPoint++, lppt++)
                            {
                            SetPixel (hdc, lppt->x, lppt->y, RGB(0, 0, 0) );
                            // DrawMarker (hdc, lppt->x, lppt->y);
                            }
                        } // if (bDrawComplete...

                    } // for (iStroke...

                }  // for (iColumn...

            }    // for (iRow...

        }      // if not NULL...

    }

/*-------------------------------------------------------------*\
 |         Handwriting Data Bounding Box Calculation.          |
\*-------------------------------------------------------------*/
PRIVATE void
hw_CalcInstanceBoundingBox(LPLIST char_instance,
                           LPRECT lpr)
    {
    INT16 i,j;
    INT16 x_min,x_max,y_min,y_max;

    x_min = y_min = MAX_VALUE;
    x_max = y_max = MIN_VALUE;

    /*----for a given char, find the maximum and minimum ------*/
    for (i = 0; i < char_instance->num_items; i++) 
        {
        LPLIST stroke = (LPLIST)char_instance->items[i];

        for (j = 0; j < stroke->num_items; j++) 
            {
            POINT p = *(LPPOINT)&stroke->items[j];

            if (p.x > x_max) x_max = p.x;
            if (p.x < x_min) x_min = p.x;
            if (p.y > y_max) y_max = p.y;
            if (p.y < y_min) y_min = p.y;
            }
        }

    lpr->left  = x_min; lpr->top    = y_min;
    lpr->right = x_max; lpr->bottom = y_max;
    }

/*-------------------------------------------------------------*\
 |                  Marker Drawing Function.                   |
\*-------------------------------------------------------------*/
void DrawMarker (HDC hdc, int x, int y)
    {
    int i;
    int rop;
    int mapmode;
    POINT pt;
    POINT ptWindowOrg;
    POINT ptViewportOrg;
    SIZE  sizeWindowExt;
    SIZE  sizeViewportExt;

    pt.x = x; pt.y = y;
    LPtoDP (hdc, &pt, 1);
    x = pt.x; y = pt.y;

    SetWindowOrgEx (hdc, 0, 0, &ptWindowOrg);
    SetViewportOrgEx (hdc, 0, 0, &ptViewportOrg);
    GetWindowExtEx (hdc, &sizeWindowExt);
    GetViewportExtEx (hdc, &sizeViewportExt);
    mapmode = SetMapMode (hdc, MM_TEXT);

    rop     = SetROP2 (hdc, R2_NOT);
    SetPixel (hdc, x, y, RGB(0, 0, 0));

    for (i=1;i<=MARKERSIZE; i++)
        {
        SetPixel (hdc, x+i, y, RGB(0, 0, 0));
        SetPixel (hdc, x-i, y, RGB(0, 0, 0));
        SetPixel (hdc, x, y+i, RGB(0, 0, 0));
        SetPixel (hdc, x, y-i, RGB(0, 0, 0));
        }

    SetROP2 (hdc, rop);

    SetMapMode (hdc, mapmode);
    SetWindowOrgEx (hdc, ptWindowOrg.x, ptWindowOrg.y, NULL);
    SetViewportOrgEx (hdc, ptViewportOrg.x, ptViewportOrg.y, NULL);
    SetWindowExtEx (hdc, sizeWindowExt.cx, sizeWindowExt.cy, NULL);
    SetViewportExtEx (hdc, sizeViewportExt.cx, sizeViewportExt.cy, NULL);
    }
